
#include "gamex-sound.hpp"
#include <windows.h>

SoundX::SoundX()
{
	samples_per_sec = 0;
	bits_per_sample = 0;
	length = 0;
	num_bytes = 0;
	num_samples = 0;	
	data = NULL;
	dx_index = -1;
	dx_refresh = true;
}

SoundX::~SoundX()
{
}

bool SoundX::ReadName (File& wav, Buffer &code, char *match_name)
{
	char buf[5];
	wav.ReadC (4, buf);
	if (strncmp (buf, match_name, 4)==0)	return true;
	else									return false;
}

bool SoundX::ReadSize (File& wav, Buffer &code, unsigned int &size)
{
	XBYTE4 fourbytes;
	wav.Read4 (fourbytes); 
	size = code.ReadL (fourbytes);
	return true;
}

bool SoundX::LoadWav (char *filename)
{
	XBYTE2 twobytes;
	XBYTE4 fourbytes;
	XBYTE2 format;
	unsigned int size;
	char buf[100];
	int n;

	File wav;
	Buffer code;

	if (wav.Open (filename, FILE_READ | FILE_RANDOM) != FILE_EXIST_NO) {
		code.SetOrder (BUFFER_ORDER_LBF);
		
		if (ReadName(wav, code, "RIFF")==false) {InvalidWav(); return false;}
		if (ReadSize(wav, code, size)==false)	{InvalidWav(); return false;}
		if (ReadName(wav, code, "WAVE")==false)	{InvalidWav(); return false;}	
		if (ReadName(wav, code, "fmt ")==false)	{InvalidWav(); return false;}
		if (ReadSize(wav, code, size)==false)	{InvalidWav(); return false;}
		// Read WAV Format Header (THIS READS 16 BYTES)
			wav.Read2 (twobytes); format = code.ReadI (twobytes);				// WAV Format Tag (1 = PCM) (IGNORED)
			if (format!=1) {wav.Close(); return false;}
			wav.Read2 (twobytes); num_channels = code.ReadI (twobytes);				// WAV Number of Channels
			wav.Read4 (fourbytes); samples_per_sec = code.ReadL (fourbytes);	// WAV Samples per Second
			wav.Read4 (fourbytes);												// WAV Avergae Bytes per Second (IGNORED)
			wav.Read2 (twobytes);												// WAV Block Align (IGNORED)
			wav.Read2 (twobytes); bits_per_sample = code.ReadI (twobytes);		// WAV Number of Channels			
			wav.ReadC (size-16, buf);
		for (; wav.EndOfFile()==0; ) {
			wav.ReadC (4, buf);
			if (strncmp (buf, "data", 4)==0) {
				if (ReadSize(wav, code, size)==false)	{InvalidWav(); return false;}
				// Read WAV Data
				num_bytes = size;								// Load number of bytes in data (from file)				
				num_samples = num_bytes / ((bits_per_sample/8) * num_channels);	// Calculate number of samples
				length = (float) num_samples / (float) samples_per_sec;		// Calculate length of sound (in seconds)
				if (data!=NULL) {delete [] data; data = NULL;}
				data = new char[num_bytes];
				if (bits_per_sample <= 8) {
					for (n = 0; n < num_bytes; n+=num_channels) 
						wav.ReadC (num_channels, data + n);
				} else if (bits_per_sample <=16) {
					for (n = 0; n < num_bytes; n+=(num_channels<<1)) 
						wav.ReadC (num_channels<<1, data + n);
				}
			} else if (strncmp (buf, "fact", 4)==0) {
				wav.Read4 (fourbytes);	
				wav.Read4 (fourbytes);			
			}
		}		
				
		dx_refresh = true;
		wav.Close ();
	} else {
		char disp[500];
		sprintf (disp, ".WAV File Not Found: %s\n\nMake sure paths and names of WAV files are correct. If you started the game outside of Visual C++, make sure the executable is in the correct location to find sounds.", filename); 
		MessageBox (NULL, disp, "GameX Error", MB_OK);
		exit(-1);
		return false;
	}	
	return true;
}

void SoundX::InvalidWav (void)
{
	char disp[500];
	sprintf (disp, ".WAV Format Invalid: \n\nInvalid WAV Format. Not all types of WAV files are supported. Try saving your WAV file with a different program, or with 'extra wav information' turned off."); 
	MessageBox (NULL, disp, "GameX Error", MB_OK);
	exit(-1);
}

void SoundX::CutLength (float new_length)
{
	int new_samples, new_bytes;
	char* new_data;

	if (data!=NULL && length > new_length) {
		new_samples = (int) (samples_per_sec * new_length);
		new_bytes = (bits_per_sample/8) * num_channels * new_samples;	
		new_data = new char[new_bytes];
		memcpy (new_data, data, new_bytes);
		if (data!=NULL) {delete [] data; data = NULL;}
		data = new_data;
		num_bytes = new_bytes;
		num_samples = new_samples;
		length = new_length;
		dx_refresh = true;
	}
}

bool SoundX::NeedUpdate (void)
{
	if (dx_index==-1 || dx_refresh==true) return true;
	return false;
}