//
// GameX (Release 2 - Vectors / Matricies / CameraX)
// Vector Class Header
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
//
//
// Vector Operations Implemented:
//		=, +, -, *, / (on vectors and scalars)
//		Cross			Cross product vector with op
//		Dot				Dot product vector with op
//		Dist (op)		Distance from vector to op
//		DistSq			Distance^2 from vector to op
//		Length ()		Length of vector
//		Normalize ()	Normalizes vector
//
// ** NOTES **
// Vector code CANNOT be inlined in header file because of dependencies
//    across vector classes (error generated: "Use of undeclared class..")
// 

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <memory.h>
#include <math.h>

#ifndef VECTOR_DEF
	#define VECTOR_DEF

											// **** Switches	
	#define VECTOR_DEBUG					// Enable Debugging Output
	#define VECTOR_INITIALIZE				// Initializes vectors	
																// Forward Referencing
	class Vector2DC;
	class Vector2DI;
	class Vector2DF;
	class Vector3DC;
	class Vector3DI;
	class Vector3DF;
	class Vector4DF;
	class MatrixF;

	// Vector2DC Declaration
	
	#define VNAME		2DC
	#define VTYPE		unsigned char

	class Vector2DC {
	public:
		VTYPE x, y;

		// Constructors/Destructors
		inline Vector2DC();
		inline ~Vector2DC();
		inline Vector2DC (VTYPE xa, VTYPE ya);
		inline Vector2DC (Vector2DC &op);	
		inline Vector2DC (Vector2DI &op);	
		inline Vector2DC (Vector2DF &op);	
		inline Vector2DC (Vector3DC &op);	
		inline Vector2DC (Vector3DI &op);	
		inline Vector2DC (Vector3DF &op);	
		inline Vector2DC (Vector4DF &op);

		// Member Functions
		inline Vector2DC &operator= (Vector2DC &op);
		inline Vector2DC &operator= (Vector2DI &op);
		inline Vector2DC &operator= (Vector2DF &op);
		inline Vector2DC &operator= (Vector3DC &op);
		inline Vector2DC &operator= (Vector3DI &op);
		inline Vector2DC &operator= (Vector3DF &op);
		inline Vector2DC &operator= (Vector4DF &op);
		
		inline Vector2DC &operator+= (Vector2DC &op);
		inline Vector2DC &operator+= (Vector2DI &op);
		inline Vector2DC &operator+= (Vector2DF &op);
		inline Vector2DC &operator+= (Vector3DC &op);
		inline Vector2DC &operator+= (Vector3DI &op);
		inline Vector2DC &operator+= (Vector3DF &op);
		inline Vector2DC &operator+= (Vector4DF &op);

		inline Vector2DC &operator-= (Vector2DC &op);
		inline Vector2DC &operator-= (Vector2DI &op);
		inline Vector2DC &operator-= (Vector2DF &op);
		inline Vector2DC &operator-= (Vector3DC &op);
		inline Vector2DC &operator-= (Vector3DI &op);
		inline Vector2DC &operator-= (Vector3DF &op);
		inline Vector2DC &operator-= (Vector4DF &op);
	
		inline Vector2DC &operator*= (Vector2DC &op);
		inline Vector2DC &operator*= (Vector2DI &op);
		inline Vector2DC &operator*= (Vector2DF &op);
		inline Vector2DC &operator*= (Vector3DC &op);
		inline Vector2DC &operator*= (Vector3DI &op);
		inline Vector2DC &operator*= (Vector3DF &op);
		inline Vector2DC &operator*= (Vector4DF &op);

		inline Vector2DC &operator/= (Vector2DC &op);
		inline Vector2DC &operator/= (Vector2DI &op);
		inline Vector2DC &operator/= (Vector2DF &op);
		inline Vector2DC &operator/= (Vector3DC &op);
		inline Vector2DC &operator/= (Vector3DI &op);
		inline Vector2DC &operator/= (Vector3DF &op);
		inline Vector2DC &operator/= (Vector4DF &op);

		// Note: Cross product does not exist for 2D vectors (only 3D)
		
		inline double Dot(Vector2DC &v);
		inline double Dot(Vector2DI &v);
		inline double Dot(Vector2DF &v);

		inline double Dist (Vector2DC &v);
		inline double Dist (Vector2DI &v);
		inline double Dist (Vector2DF &v);
		inline double Dist (Vector3DC &v);
		inline double Dist (Vector3DI &v);
		inline double Dist (Vector3DF &v);
		inline double Dist (Vector4DF &v);

		inline double DistSq (Vector2DC &v);		
		inline double DistSq (Vector2DI &v);		
		inline double DistSq (Vector2DF &v);		
		inline double DistSq (Vector3DC &v);		
		inline double DistSq (Vector3DI &v);		
		inline double DistSq (Vector3DF &v);		
		inline double DistSq (Vector4DF &v);

		inline Vector2DC &Normalize (void);
		inline double Length (void);

		inline VTYPE &X(void);
		inline VTYPE &Y(void);
		inline VTYPE Z(void);
		inline VTYPE W(void);
		inline const VTYPE &X(void) const;			
		inline const VTYPE &Y(void) const;			
		inline const VTYPE Z(void) const;			
		inline const VTYPE W(void) const;
		inline VTYPE *Data (void);
	};
	
	#undef VNAME
	#undef VTYPE

	// Vector2DI Declaration

	#define VNAME		2DI
	#define VTYPE		int

	class Vector2DI {
	public:
		VTYPE x, y;

		// Constructors/Destructors
		inline Vector2DI();							
		inline ~Vector2DI();			
		inline Vector2DI (VTYPE xa, VTYPE ya);
		inline Vector2DI (Vector2DC &op);				
		inline Vector2DI (Vector2DI &op);				
		inline Vector2DI (Vector2DF &op);				
		inline Vector2DI (Vector3DC &op);				
		inline Vector2DI (Vector3DI &op);				
		inline Vector2DI (Vector3DF &op);				
		inline Vector2DI (Vector4DF &op);

		// Member Functions
		inline Vector2DI &operator= (Vector2DC &op);
		inline Vector2DI &operator= (Vector2DI &op);
		inline Vector2DI &operator= (Vector2DF &op);
		inline Vector2DI &operator= (Vector3DC &op);
		inline Vector2DI &operator= (Vector3DI &op);
		inline Vector2DI &operator= (Vector3DF &op);
		inline Vector2DI &operator= (Vector4DF &op);

		inline Vector2DI &operator+= (Vector2DC &op);
		inline Vector2DI &operator+= (Vector2DI &op);
		inline Vector2DI &operator+= (Vector2DF &op);
		inline Vector2DI &operator+= (Vector3DC &op);
		inline Vector2DI &operator+= (Vector3DI &op);
		inline Vector2DI &operator+= (Vector3DF &op);
		inline Vector2DI &operator+= (Vector4DF &op);

		inline Vector2DI &operator-= (Vector2DC &op);
		inline Vector2DI &operator-= (Vector2DI &op);
		inline Vector2DI &operator-= (Vector2DF &op);
		inline Vector2DI &operator-= (Vector3DC &op);
		inline Vector2DI &operator-= (Vector3DI &op);
		inline Vector2DI &operator-= (Vector3DF &op);
		inline Vector2DI &operator-= (Vector4DF &op);
	
		inline Vector2DI &operator*= (Vector2DC &op);
		inline Vector2DI &operator*= (Vector2DI &op);
		inline Vector2DI &operator*= (Vector2DF &op);
		inline Vector2DI &operator*= (Vector3DC &op);
		inline Vector2DI &operator*= (Vector3DI &op);
		inline Vector2DI &operator*= (Vector3DF &op);
		inline Vector2DI &operator*= (Vector4DF &op);

		inline Vector2DI &operator/= (Vector2DC &op);
		inline Vector2DI &operator/= (Vector2DI &op);
		inline Vector2DI &operator/= (Vector2DF &op);
		inline Vector2DI &operator/= (Vector3DC &op);
		inline Vector2DI &operator/= (Vector3DI &op);
		inline Vector2DI &operator/= (Vector3DF &op);
		inline Vector2DI &operator/= (Vector4DF &op);


		// Note: Cross product does not exist for 2D vectors (only 3D)
		
		inline double Dot(Vector2DC &v);
		inline double Dot(Vector2DI &v);
		inline double Dot(Vector2DF &v);

		inline double Dist (Vector2DC &v);
		inline double Dist (Vector2DI &v);
		inline double Dist (Vector2DF &v);
		inline double Dist (Vector3DC &v);
		inline double Dist (Vector3DI &v);
		inline double Dist (Vector3DF &v);
		inline double Dist (Vector4DF &v);

		inline double DistSq (Vector2DC &v);
		inline double DistSq (Vector2DI &v);
		inline double DistSq (Vector2DF &v);
		inline double DistSq (Vector3DC &v);
		inline double DistSq (Vector3DI &v);
		inline double DistSq (Vector3DF &v);
		inline double DistSq (Vector4DF &v);
		
		inline Vector2DI &Normalize (void);
		inline double Length (void);

		inline VTYPE &X(void);
		inline VTYPE &Y(void);
		inline VTYPE Z(void);
		inline VTYPE W(void);
		inline const VTYPE &X(void) const;
		inline const VTYPE &Y(void) const;
		inline const VTYPE Z(void) const;
		inline const VTYPE W(void) const;
		inline VTYPE *Data (void);
	};
	
	#undef VNAME
	#undef VTYPE

	// Vector2DF Declarations

	#define VNAME		2DF
	#define VTYPE		double

	class Vector2DF {
	public:
		VTYPE x, y;

		// Constructors/Destructors
		inline Vector2DF ();
		inline ~Vector2DF ();
		inline Vector2DF (VTYPE xa, VTYPE ya);
		inline Vector2DF (Vector2DC &op);
		inline Vector2DF (Vector2DI &op);
		inline Vector2DF (Vector2DF &op);
		inline Vector2DF (Vector3DC &op);
		inline Vector2DF (Vector3DI &op);
		inline Vector2DF (Vector3DF &op);
		inline Vector2DF (Vector4DF &op);

		// Member Functions
		inline Vector2DF &operator= (Vector2DC &op);
		inline Vector2DF &operator= (Vector2DI &op);
		inline Vector2DF &operator= (Vector2DF &op);
		inline Vector2DF &operator= (Vector3DC &op);
		inline Vector2DF &operator= (Vector3DI &op);
		inline Vector2DF &operator= (Vector3DF &op);
		inline Vector2DF &operator= (Vector4DF &op);
		
		inline Vector2DF &operator+= (Vector2DC &op);
		inline Vector2DF &operator+= (Vector2DI &op);
		inline Vector2DF &operator+= (Vector2DF &op);
		inline Vector2DF &operator+= (Vector3DC &op);
		inline Vector2DF &operator+= (Vector3DI &op);
		inline Vector2DF &operator+= (Vector3DF &op);
		inline Vector2DF &operator+= (Vector4DF &op);

		inline Vector2DF &operator-= (Vector2DC &op);
		inline Vector2DF &operator-= (Vector2DI &op);
		inline Vector2DF &operator-= (Vector2DF &op);
		inline Vector2DF &operator-= (Vector3DC &op);
		inline Vector2DF &operator-= (Vector3DI &op);
		inline Vector2DF &operator-= (Vector3DF &op);
		inline Vector2DF &operator-= (Vector4DF &op);

		inline Vector2DF &operator*= (Vector2DC &op);
		inline Vector2DF &operator*= (Vector2DI &op);
		inline Vector2DF &operator*= (Vector2DF &op);
		inline Vector2DF &operator*= (Vector3DC &op);
		inline Vector2DF &operator*= (Vector3DI &op);
		inline Vector2DF &operator*= (Vector3DF &op);
		inline Vector2DF &operator*= (Vector4DF &op);

		inline Vector2DF &operator/= (Vector2DC &op);
		inline Vector2DF &operator/= (Vector2DI &op);
		inline Vector2DF &operator/= (Vector2DF &op);
		inline Vector2DF &operator/= (Vector3DC &op);
		inline Vector2DF &operator/= (Vector3DI &op);
		inline Vector2DF &operator/= (Vector3DF &op);
		inline Vector2DF &operator/= (Vector4DF &op);

		// Note: Cross product does not exist for 2D vectors (only 3D)
		
		inline double Dot(Vector2DC &v);
		inline double Dot(Vector2DI &v);
		inline double Dot(Vector2DF &v);

		inline double Dist (Vector2DC &v);
		inline double Dist (Vector2DI &v);
		inline double Dist (Vector2DF &v);
		inline double Dist (Vector3DC &v);
		inline double Dist (Vector3DI &v);
		inline double Dist (Vector3DF &v);
		inline double Dist (Vector4DF &v);

		inline double DistSq (Vector2DC &v);
		inline double DistSq (Vector2DI &v);
		inline double DistSq (Vector2DF &v);
		inline double DistSq (Vector3DC &v);
		inline double DistSq (Vector3DI &v);
		inline double DistSq (Vector3DF &v);
		inline double DistSq (Vector4DF &v);

		inline Vector2DF &Normalize (void);
		inline double Length (void);

		inline VTYPE &X(void);
		inline VTYPE &Y(void);
		inline VTYPE Z(void);
		inline VTYPE W(void);
		inline const VTYPE &X(void) const;
		inline const VTYPE &Y(void) const;
		inline const VTYPE Z(void) const;
		inline const VTYPE W(void) const;
		inline VTYPE *Data (void);
	};
	
	#undef VNAME
	#undef VTYPE

	// Vector3DC Declaration
	
	#define VNAME		3DC
	#define VTYPE		unsigned char

	class Vector3DC {
	public:	
		VTYPE x, y, z;
	
		// Constructors/Destructors
		inline Vector3DC();
		inline ~Vector3DC();
		inline Vector3DC (VTYPE xa, VTYPE ya, VTYPE za);
		inline Vector3DC (Vector2DC &op);
		inline Vector3DC (Vector2DI &op);
		inline Vector3DC (Vector2DF &op);
		inline Vector3DC (Vector3DC &op);
		inline Vector3DC (Vector3DI &op);
		inline Vector3DC (Vector3DF &op);
		inline Vector3DC (Vector4DF &op);

		// Member Functions
		inline Vector3DC &Set (VTYPE xa, VTYPE ya, VTYPE za);
		
		inline Vector3DC &operator= (Vector2DC &op);
		inline Vector3DC &operator= (Vector2DI &op);
		inline Vector3DC &operator= (Vector2DF &op);
		inline Vector3DC &operator= (Vector3DC &op);
		inline Vector3DC &operator= (Vector3DI &op);
		inline Vector3DC &operator= (Vector3DF &op);
		inline Vector3DC &operator= (Vector4DF &op);
		
		inline Vector3DC &operator+= (Vector2DC &op);
		inline Vector3DC &operator+= (Vector2DI &op);
		inline Vector3DC &operator+= (Vector2DF &op);
		inline Vector3DC &operator+= (Vector3DC &op);
		inline Vector3DC &operator+= (Vector3DI &op);
		inline Vector3DC &operator+= (Vector3DF &op);
		inline Vector3DC &operator+= (Vector4DF &op);

		inline Vector3DC &operator-= (Vector2DC &op);
		inline Vector3DC &operator-= (Vector2DI &op);
		inline Vector3DC &operator-= (Vector2DF &op);
		inline Vector3DC &operator-= (Vector3DC &op);
		inline Vector3DC &operator-= (Vector3DI &op);
		inline Vector3DC &operator-= (Vector3DF &op);
		inline Vector3DC &operator-= (Vector4DF &op);
	
		inline Vector3DC &operator*= (Vector2DC &op);
		inline Vector3DC &operator*= (Vector2DI &op);
		inline Vector3DC &operator*= (Vector2DF &op);
		inline Vector3DC &operator*= (Vector3DC &op);
		inline Vector3DC &operator*= (Vector3DI &op);
		inline Vector3DC &operator*= (Vector3DF &op);
		inline Vector3DC &operator*= (Vector4DF &op);

		inline Vector3DC &operator/= (Vector2DC &op);
		inline Vector3DC &operator/= (Vector2DI &op);
		inline Vector3DC &operator/= (Vector2DF &op);
		inline Vector3DC &operator/= (Vector3DC &op);
		inline Vector3DC &operator/= (Vector3DI &op);
		inline Vector3DC &operator/= (Vector3DF &op);
		inline Vector3DC &operator/= (Vector4DF &op);

		inline Vector3DC &Cross (Vector3DC &v);
		inline Vector3DC &Cross (Vector3DI &v);
		inline Vector3DC &Cross (Vector3DF &v);	
		
		inline double Dot(Vector3DC &v);
		inline double Dot(Vector3DI &v);
		inline double Dot(Vector3DF &v);

		inline double Dist (Vector2DC &v);
		inline double Dist (Vector2DI &v);
		inline double Dist (Vector2DF &v);
		inline double Dist (Vector3DC &v);
		inline double Dist (Vector3DI &v);
		inline double Dist (Vector3DF &v);
		inline double Dist (Vector4DF &v);

		inline double DistSq (Vector2DC &v);
		inline double DistSq (Vector2DI &v);
		inline double DistSq (Vector2DF &v);
		inline double DistSq (Vector3DC &v);
		inline double DistSq (Vector3DI &v);
		inline double DistSq (Vector3DF &v);
		inline double DistSq (Vector4DF &v);

		inline Vector3DC &Normalize (void);
		inline double Length (void);

		inline VTYPE &X(void);
		inline VTYPE &Y(void);
		inline VTYPE &Z(void);
		inline VTYPE W(void);
		inline const VTYPE &X(void) const;
		inline const VTYPE &Y(void) const;
		inline const VTYPE &Z(void) const;
		inline const VTYPE W(void) const;
		inline VTYPE *Data (void);
	};
	
	#undef VNAME
	#undef VTYPE

	// Vector3DI Declaration

	#define VNAME		3DI
	#define VTYPE		int

	class Vector3DI {
	public:
		VTYPE x, y, z;
	
		// Constructors/Destructors
		inline Vector3DI();
		inline ~Vector3DI();
		inline Vector3DI (VTYPE xa, VTYPE ya, VTYPE za);
		inline Vector3DI (Vector2DC &op);
		inline Vector3DI (Vector2DI &op);
		inline Vector3DI (Vector2DF &op);
		inline Vector3DI (Vector3DC &op);
		inline Vector3DI (Vector3DI &op);
		inline Vector3DI (Vector3DF &op);
		inline Vector3DI (Vector4DF &op);

		// Member Functions
		inline Vector3DI &operator= (Vector2DC &op);
		inline Vector3DI &operator= (Vector2DI &op);
		inline Vector3DI &operator= (Vector2DF &op);
		inline Vector3DI &operator= (Vector3DC &op);
		inline Vector3DI &operator= (Vector3DI &op);
		inline Vector3DI &operator= (Vector3DF &op);
		inline Vector3DI &operator= (Vector4DF &op);
		
		inline Vector3DI &operator+= (Vector2DC &op);
		inline Vector3DI &operator+= (Vector2DI &op);
		inline Vector3DI &operator+= (Vector2DF &op);
		inline Vector3DI &operator+= (Vector3DC &op);
		inline Vector3DI &operator+= (Vector3DI &op);
		inline Vector3DI &operator+= (Vector3DF &op);
		inline Vector3DI &operator+= (Vector4DF &op);

		inline Vector3DI &operator-= (Vector2DC &op);
		inline Vector3DI &operator-= (Vector2DI &op);
		inline Vector3DI &operator-= (Vector2DF &op);
		inline Vector3DI &operator-= (Vector3DC &op);
		inline Vector3DI &operator-= (Vector3DI &op);
		inline Vector3DI &operator-= (Vector3DF &op);
		inline Vector3DI &operator-= (Vector4DF &op);
	
		inline Vector3DI &operator*= (Vector2DC &op);
		inline Vector3DI &operator*= (Vector2DI &op);
		inline Vector3DI &operator*= (Vector2DF &op);
		inline Vector3DI &operator*= (Vector3DC &op);
		inline Vector3DI &operator*= (Vector3DI &op);
		inline Vector3DI &operator*= (Vector3DF &op);
		inline Vector3DI &operator*= (Vector4DF &op);

		inline Vector3DI &operator/= (Vector2DC &op);
		inline Vector3DI &operator/= (Vector2DI &op);
		inline Vector3DI &operator/= (Vector2DF &op);
		inline Vector3DI &operator/= (Vector3DC &op);
		inline Vector3DI &operator/= (Vector3DI &op);
		inline Vector3DI &operator/= (Vector3DF &op);
		inline Vector3DI &operator/= (Vector4DF &op);

		inline Vector3DI &Cross (Vector3DC &v);
		inline Vector3DI &Cross (Vector3DI &v);
		inline Vector3DI &Cross (Vector3DF &v);	
		
		inline double Dot(Vector3DC &v);
		inline double Dot(Vector3DI &v);
		inline double Dot(Vector3DF &v);

		inline double Dist (Vector2DC &v);
		inline double Dist (Vector2DI &v);
		inline double Dist (Vector2DF &v);
		inline double Dist (Vector3DC &v);
		inline double Dist (Vector3DI &v);
		inline double Dist (Vector3DF &v);
		inline double Dist (Vector4DF &v);

		inline double DistSq (Vector2DC &v);
		inline double DistSq (Vector2DI &v);
		inline double DistSq (Vector2DF &v);
		inline double DistSq (Vector3DC &v);
		inline double DistSq (Vector3DI &v);
		inline double DistSq (Vector3DF &v);
		inline double DistSq (Vector4DF &v);

		inline Vector3DI &Normalize (void);
		inline double Length (void);

		inline VTYPE &X(void);
		inline VTYPE &Y(void);
		inline VTYPE &Z(void);
		inline VTYPE W(void);
		inline const VTYPE &X(void) const;
		inline const VTYPE &Y(void) const;
		inline const VTYPE &Z(void) const;
		inline const VTYPE W(void) const;
		inline VTYPE *Data (void);
	};
	
	#undef VNAME
	#undef VTYPE

	// Vector3DF Declarations

	#define VNAME		3DF
	#define VTYPE		double

	class Vector3DF {
	public:
		VTYPE x, y, z;
	
		// Constructors/Destructors
		inline Vector3DF();
		inline ~Vector3DF();
		inline Vector3DF (VTYPE xa, VTYPE ya, VTYPE za);
		inline Vector3DF (Vector2DC &op);
		inline Vector3DF (Vector2DI &op);
		inline Vector3DF (Vector2DF &op);
		inline Vector3DF (Vector3DC &op);
		inline Vector3DF (Vector3DI &op);
		inline Vector3DF (Vector3DF &op);
		inline Vector3DF (Vector4DF &op);

		// Set Functions
		inline Vector3DF &Set (double xa, double ya, double za);
		
		// Member Functions
		inline Vector3DF &operator= (int op);
		inline Vector3DF &operator= (double op);
		inline Vector3DF &operator= (Vector2DC &op);
		inline Vector3DF &operator= (Vector2DI &op);
		inline Vector3DF &operator= (Vector2DF &op);
		inline Vector3DF &operator= (Vector3DC &op);
		inline Vector3DF &operator= (Vector3DI &op);
		inline Vector3DF &operator= (Vector3DF &op);
		inline Vector3DF &operator= (Vector4DF &op);

		inline Vector3DF &operator+= (int op);
		inline Vector3DF &operator+= (double op);
		inline Vector3DF &operator+= (Vector2DC &op);
		inline Vector3DF &operator+= (Vector2DI &op);
		inline Vector3DF &operator+= (Vector2DF &op);
		inline Vector3DF &operator+= (Vector3DC &op);
		inline Vector3DF &operator+= (Vector3DI &op);
		inline Vector3DF &operator+= (Vector3DF &op);
		inline Vector3DF &operator+= (Vector4DF &op);

		inline Vector3DF &operator-= (int op);
		inline Vector3DF &operator-= (double op);
		inline Vector3DF &operator-= (Vector2DC &op);
		inline Vector3DF &operator-= (Vector2DI &op);
		inline Vector3DF &operator-= (Vector2DF &op);
		inline Vector3DF &operator-= (Vector3DC &op);
		inline Vector3DF &operator-= (Vector3DI &op);
		inline Vector3DF &operator-= (Vector3DF &op);
		inline Vector3DF &operator-= (Vector4DF &op);
	
		inline Vector3DF &operator*= (int op);
		inline Vector3DF &operator*= (double op);
		inline Vector3DF &operator*= (Vector2DC &op);
		inline Vector3DF &operator*= (Vector2DI &op);
		inline Vector3DF &operator*= (Vector2DF &op);
		inline Vector3DF &operator*= (Vector3DC &op);
		inline Vector3DF &operator*= (Vector3DI &op);
		inline Vector3DF &operator*= (Vector3DF &op);
		inline Vector3DF &operator*= (Vector4DF &op);
		Vector3DF &operator*= (MatrixF &op);				// see vector.cpp

		inline Vector3DF &operator/= (int op);
		inline Vector3DF &operator/= (double op);
		inline Vector3DF &operator/= (Vector2DC &op);
		inline Vector3DF &operator/= (Vector2DI &op);
		inline Vector3DF &operator/= (Vector2DF &op);
		inline Vector3DF &operator/= (Vector3DC &op);
		inline Vector3DF &operator/= (Vector3DI &op);
		inline Vector3DF &operator/= (Vector3DF &op);
		inline Vector3DF &operator/= (Vector4DF &op);

		inline Vector3DF &Cross (Vector3DC &v);
		inline Vector3DF &Cross (Vector3DI &v);
		inline Vector3DF &Cross (Vector3DF &v);	
		
		inline double Dot(Vector3DC &v);
		inline double Dot(Vector3DI &v);
		inline double Dot(Vector3DF &v);

		inline double Dist (Vector2DC &v);
		inline double Dist (Vector2DI &v);
		inline double Dist (Vector2DF &v);
		inline double Dist (Vector3DC &v);
		inline double Dist (Vector3DI &v);
		inline double Dist (Vector3DF &v);
		inline double Dist (Vector4DF &v);

		inline double DistSq (Vector2DC &v);
		inline double DistSq (Vector2DI &v);
		inline double DistSq (Vector2DF &v);
		inline double DistSq (Vector3DC &v);
		inline double DistSq (Vector3DI &v);
		inline double DistSq (Vector3DF &v);
		inline double DistSq (Vector4DF &v);

		inline Vector3DF &Normalize (void);
		inline double Length (void);

		inline VTYPE &X(void);
		inline VTYPE &Y(void);
		inline VTYPE &Z(void);
		inline VTYPE W(void);
		inline const VTYPE &X(void) const;
		inline const VTYPE &Y(void) const;
		inline const VTYPE &Z(void) const;
		inline const VTYPE W(void) const;
		inline VTYPE *Data (void);
	};
	
	#undef VNAME
	#undef VTYPE

	// Vector4DF Declarations

	#define VNAME		4DF
	#define VTYPE		double

	class Vector4DF {
	public:
		VTYPE x, y, z, w;
	
		// Constructors/Destructors
		inline Vector4DF();
		inline ~Vector4DF();
		inline Vector4DF (VTYPE xa, VTYPE ya, VTYPE za, VTYPE wa);
		inline Vector4DF (Vector2DC &op);
		inline Vector4DF (Vector2DI &op);
		inline Vector4DF (Vector2DF &op);
		inline Vector4DF (Vector3DC &op);
		inline Vector4DF (Vector3DI &op);
		inline Vector4DF (Vector3DF &op);
		inline Vector4DF (Vector4DF &op);

		// Member Functions
		inline Vector4DF &operator= (int op);
		inline Vector4DF &operator= (double op);
		inline Vector4DF &operator= (Vector2DC &op);
		inline Vector4DF &operator= (Vector2DI &op);
		inline Vector4DF &operator= (Vector2DF &op);
		inline Vector4DF &operator= (Vector3DC &op);
		inline Vector4DF &operator= (Vector3DI &op);
		inline Vector4DF &operator= (Vector3DF &op);
		inline Vector4DF &operator= (Vector4DF &op);

		inline Vector4DF &operator+= (int op);
		inline Vector4DF &operator+= (double op);
		inline Vector4DF &operator+= (Vector2DC &op);
		inline Vector4DF &operator+= (Vector2DI &op);
		inline Vector4DF &operator+= (Vector2DF &op);
		inline Vector4DF &operator+= (Vector3DC &op);
		inline Vector4DF &operator+= (Vector3DI &op);
		inline Vector4DF &operator+= (Vector3DF &op);
		inline Vector4DF &operator+= (Vector4DF &op);

		inline Vector4DF &operator-= (int op);
		inline Vector4DF &operator-= (double op);
		inline Vector4DF &operator-= (Vector2DC &op);
		inline Vector4DF &operator-= (Vector2DI &op);
		inline Vector4DF &operator-= (Vector2DF &op);
		inline Vector4DF &operator-= (Vector3DC &op);
		inline Vector4DF &operator-= (Vector3DI &op);
		inline Vector4DF &operator-= (Vector3DF &op);
		inline Vector4DF &operator-= (Vector4DF &op);
	
		inline Vector4DF &operator*= (int op);
		inline Vector4DF &operator*= (double op);
		inline Vector4DF &operator*= (Vector2DC &op);
		inline Vector4DF &operator*= (Vector2DI &op);
		inline Vector4DF &operator*= (Vector2DF &op);
		inline Vector4DF &operator*= (Vector3DC &op);
		inline Vector4DF &operator*= (Vector3DI &op);
		inline Vector4DF &operator*= (Vector3DF &op);
		inline Vector4DF &operator*= (Vector4DF &op);
		Vector4DF &operator*= (MatrixF &op);				// see vector.cpp

		inline Vector4DF &operator/= (int op);
		inline Vector4DF &operator/= (double op);
		inline Vector4DF &operator/= (Vector2DC &op);
		inline Vector4DF &operator/= (Vector2DI &op);
		inline Vector4DF &operator/= (Vector2DF &op);
		inline Vector4DF &operator/= (Vector3DC &op);
		inline Vector4DF &operator/= (Vector3DI &op);
		inline Vector4DF &operator/= (Vector3DF &op);
		inline Vector4DF &operator/= (Vector4DF &op);

		inline Vector4DF &Cross (Vector4DF &v);	
		
		inline double Dot(Vector4DF &v);

		inline double Dist (Vector4DF &v);

		inline double DistSq (Vector4DF &v);

		inline Vector4DF &Normalize (void);
		inline double Length (void);

		inline VTYPE &X(void);
		inline VTYPE &Y(void);
		inline VTYPE &Z(void);
		inline VTYPE &W(void);
		inline const VTYPE &X(void) const;
		inline const VTYPE &Y(void) const;
		inline const VTYPE &Z(void) const;
		inline const VTYPE &W(void) const;
		inline VTYPE *Data (void);
	};
	
	#undef VNAME
	#undef VTYPE



    // Vector Code Definitions (Inlined)

	#include "gamex-vector.cci"

#endif